@@ -10,7 +10,6 @@
 // Own
 #include "NullProcessInfo.h"
 #include "ProcessInfo.h"
-#include "SSHProcessInfo.h"
 #include "UnixProcessInfo.h"
 
 // Unix
@@ -135,7 +134,7 @@ QStringList ProcessInfo::commonDirNames()
 
     if (forTheFirstTime) {
         const KSharedConfigPtr &config = KSharedConfig::openConfig();
-        const KConfigGroup &configGroup = config->group("ProcessInfo");
+        const KConfigGroup &configGroup = config->group(QStringLiteral("ProcessInfo"));
         // Need to make a local copy so the begin() and end() point to the same QList
         _commonDirNames = configGroup.readEntry("CommonDirNames", QStringList());
         _commonDirNames.removeDuplicates();
