@@ -10,12 +10,7 @@
 
 #include <QtGlobal>
 // KDE
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
 #include <KNSCore/Entry>
-#else
-#include <KNS3/Entry>
-#include <KNSCore/EntryInternal>
-#endif
 #include <KPageDialog>
 #include <knewstuff_version.h>
 
@@ -136,6 +131,8 @@ private Q_SLOTS:
     void showTerminalSizeHint(bool);
     void setDimWhenInactive(bool);
     void setDimValue(int value);
+    void setBorderWhenActive(bool);
+    void setFocusBorderColor(const QColor &color);
     void showEnvironmentEditor();
 
     // appearance page
@@ -148,6 +145,8 @@ private Q_SLOTS:
     void removeColorScheme();
     void setVerticalLine(bool);
     void setVerticalLineColumn(int);
+    // void focusBorderColor();
+    void focusBorderColorChanged(const QColor &color);
     void toggleBlinkingCursor(bool);
     void setCursorShape(int);
     void autoCursorColor();
@@ -161,15 +160,7 @@ private Q_SLOTS:
     void togglebidiTableDirOverride(bool);
     void togglebidiLineLTR(bool);
 
-#if KNEWSTUFF_VERSION >= QT_VERSION_CHECK(5, 91, 0)
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
     void gotNewColorSchemes(const QList<KNSCore::Entry> &changedEntries);
-#else
-    void gotNewColorSchemes(const QList<KNSCore::EntryInternal> &changedEntries);
-#endif
-#else
-    void gotNewColorSchemes(const KNS3::Entry::List &changedEntries);
-#endif
 
     /**
      * Deletes the selected colorscheme from the user's home dir location
